# noinspection PyUnresolvedReferences
from .confirm import send_email, send_password, verify_email, verify_password, verify_token, verify_email_view, \
    verify_password_view, verify_view
# noinspection PyUnresolvedReferences
from .views import verify_email_page, verify_password_page
